//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//
//  "S I M P L E S T A T I S T I C S " plugin
//
//	Collects and lists some statistics about the current score.
//	Version 2.0 - Date 11.8.2019. 
//
//	Refered "Score Statistics" By Maurizio M. Gavioli, 2010 and Joachim Schmitz, 2012. 
//
//  By Snowwolf, 2019. 
//
//  MuseScore: Copyright (C)2008 Werner Schweer and others
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================


import QtQuick 2.0
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import MuseScore 1.0
import FileIO 1.0

MuseScore {
   version: "2.0"
   description: "This plugin Statistic Score"
   menuPath: "Plugins.ScoreStat"
   pluginType: "dialog" 
   width:  600
   height: 300
   

//---------------------------------------------------------
//	init()
//	this function will be called on startup of mscore
//---------------------------------------------------------

//------------output_Variables------------------
// Arrays containing the number of occurrences and the total duration of each pitch
property var  g_numOfPitches : [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
property var  g_lenOfPitches : [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];

//// Arrays containing the occurrences of each note duration and of each rest duration
property var  g_noteLengths : [];
property var  g_restLengths : [];

// Array with the note names used for output
property var  g_noteNames : ["C", "C#/Db", "D", "D#/Eb", "E", "F", "F#/Gb", "G", "G#/Ab", "A", "A#/Bb", "B"];

// variables with totals
property var  g_numOfPages : 0;
property var  g_numOfBars  : 0;
property var  g_numOfParts : 0;
property var  g_lenOfNotes : 0;
property var  g_lenOfRests : 0;
property var  g_numOfNotes : 0;
property var  g_numOfRests : 0;
//------------------------------------------------



//-------------------------------------------------------------------
//	run()
//	this function will be called when activating the plugin menu entry
//
//	global Variables:
//	pluginPath - contains the plugin path; file separator is "/"
//-------------------------------------------------------------------

//-------------------------------------------------------------------
//Excution order: Rectangle to Text to MouseArea.OnClick, where do text=statistics(), it will use statistics function. 
//To do: let Statistics just assign output variable, let Rectagle get variable calculated already. 
//-------------------------------------------------------------------



onRun:
{

}

   Rectangle {
        id:rect1 
        color: "grey"
        anchors.fill: parent 
        width:  600
        height: 300

        Text {
            id: txt1
            anchors.centerIn: parent
            text:qsTr("Click Me")
            }

        MouseArea {
            anchors.fill: parent
            onClicked: 
            {
            
            txt1.text = qsTr(statistics());
            console.log(g_lenOfNotes);
            //Qt.quit()
            }
            }
        }
        
function statistics()
{

//------------working varibles-----------------
	var		chordnote, staff, voice;
	var		cursor;
	var		idx;
	var		len, note, pitch;
	var		numOfChordNotes;
//----------------------------------------------



	// determine version
	if( this.mscoreVersion == undefined)
	{	QMessageBox.critical(null, "Unsupported version", "");
		return;
	}
	// no score open (MuseScore 2.0+, can't happen earlier)
	if (typeof curScore === 'undefined')
		return;
	// score totals (not implemented in some older MuseScore versions)
	if(curScore.parts != undefined)
		g_numOfParts	= curScore.parts;
	if(curScore.pages != undefined)
		g_numOfPages	= curScore.pages;
	if(curScore.measures != undefined)
		g_numOfBars		= curScore.measures;
	cursor = curScore.newCursor();
	// for each cursor step of each voice of each staff

	for (staff = 0; staff < curScore.nstaves; ++staff)
	{	cursor.staffIdx = staff;
		for (voice = 0; voice < 4; voice++)
		{	
                    cursor.voice = voice;
			cursor.rewind(0);  // set cursor to first chord/rest
                        var flag = true; 
			while (flag)
			{	// if cursor is at a chord, get chord duration
                      if(cursor.element!=null)
                         {
				if (cursor.element.type==Element.CHORD)
				{	len			= cursor.element.duration;
					numOfChordNotes	= cursor.element.notes.length;
					g_numOfNotes += numOfChordNotes;
					// enumerate chord notes and get the pitch and the duration of each
					for (chordnote = 0; chordnote < numOfChordNotes; chordnote++)
					{	note	= cursor.element.notes[chordnote];
						pitch	= note.pitch % 12;
						g_numOfPitches[pitch]++;
						g_lenOfPitches[pitch] += len.numerator/len.denominator;
						g_lenOfNotes += len.numerator/len.denominator;
						
					}
				}
				// if cursor is at a rest, get its duration
				else if (cursor.element.type==Element.REST)
				{	g_numOfRests++;
					len = cursor.element.duration;
					g_lenOfRests += len.numerator/len.denominator;
					
				}
                         }
				flag = cursor.next();

			}
		}
	}
	//outtxt is what log out in the box
	var outtxt="Name:"+g_noteNames.toString()+"\n"+"Num:"+g_numOfPitches.toString()+"\n"+"length:"+g_lenOfPitches.toString();//+"\n"+ "NoteLength:"+g_noteLengths.toString()+"\n"+"RestLength:"+g_restLengths.toString()+"\n"+"Pages:"+g_numOfPages.toString()+"\n"+"Pages:"+g_numOfBars.toString()+"\n"+"Pages:"+g_numOfParts.toString();
      return outtxt;
      
}
        

}
